=begin
**
** commands.rb
** 30/JUL/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Main file of the Commands namespace. Requires for other sub modules should
** be included here.
** 
** The SoapDriver module is included here, mainly because It does not fit
** elsewhere.
**
** Version:
**  v1.0 [30/July/2007]: first released
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

# Commands module groups three concepts: *parsers*, *dispatchers* and *modules*.
#
# = Parsers
# A parser is an object that receives user input (lines of text) and forwards 
# the request to the corresponding dispatcher. See Commands::Parser.
# 
# = Dispatchers
# A dispatcher is an object that can handle user request and perform actions.
# There is no limit in the amount or nature that a dispatcher can perform. See
# Dispatchers.
#
# = Modules
# Modules are bits of code that can be plugged in into +dradis+ to provide
# specific functionality. 
#
# Each module must inherit from a dispatcher. Dispatchers must be seen as a
# general class of module while Modules are concrete implementations providing 
# functionality.
#
# +dradis+ core modules are:
# - Add: provides with commands to add new information to the
#   knowledge base of the server.
# - Del: operations to remove information from the server knowledge 
#   base.
# - Show: includes actions to query information already stored on the 
#   server.
# - Soap: contains SOAP operations that do not fit in the
#   Add/Del/Show categories
# - Filesystem: demo module that exposes some functionts to interact
#   with the underlying filesystem.
# - Chucknorris: shows a random Chuck Norris fact. This module is 
#   based on a similar module of *rbot* project. Please visit http://linuxbrit.co.uk/rbot/
#   for further details.
#
module Commands    
  
  # SoapDriver is meant to be used as a mixin for every Dispatcher or 
  # NamespaceDispatcher that requires SOAP operations with the +dradis+
  # server.
  # 
  # The module provides with a @*driver* variable to the including code. This
  # variable can be used to access remote SOAP services.
  module SoapDriver
    
    # Get the driver ready to perform a new query. If @*driver* has not been 
    # initialized this is done, otherwise the driver is resetted.
    def setup_soap()
      if ($driver == nil)
        wsdlfile = $config.get_option(:wsdluri)
        $driver = SOAP::WSDLDriverFactory.new(wsdlfile).create_rpc_driver
      end
      if @driver == nil
        @driver = $driver
      else
        @driver.reset_stream
      end
    end
  end
end

require 'commands/dispatcher'
require 'commands/parser'
