=begin
**
** del.rb
** 18/JUN/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   This module uses the SOAP interface to remove information from the knowledge
** base of the dradis server.
**
** Version:
**  v1.0 [18/June/2007]: first released
**  v1.1 [24/October/2007]: return a formated output insted of a SOAP mapping object
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Commands

  # This module uses the SoapDriver to remove information from the knowledge
  # base of the +dradis+ server. Actions provided are:
  # [+host+] remove a new host
  # [+protocol+] remove a new IP protocol
  # [+service+] remove a new service for a host on a given port
  # [+category+] remove a new note category  
  class Del < Dispatchers::Namespace
    include SoapDriver

 
    #----------------------------------------------------- private methods
    private
    INFO = { 
      :namespace => 'del',
      :description => 'remove information from the server knowledge base',
      :commands => {
        'host' => {
          :desc => 'new host',
          :syntax => [
            {
              :required => true,
              :label=>'host_id', 
              :regexp=> /\d/
            }          
          ]
        },
        'protocol' => {
          :desc => 'remove a protocol from the server',
          :syntax => [
            {
              :required => true,
              :label=>'protocol_name', 
              :regexp=> /\w+/
            }          
          ]
        },
        'service' => {
          :desc => 'remove a service from a host',
          :syntax => [
            {
              :required => true,
              :label=>'service_id', 
              :regexp=> /\d/
            }                    
          ]
        },
        'category' => {
          :desc => 'remove a note category',
          :syntax => [
            {
              :required => true,
              :label=>'category_name', 
              :regexp=> /\w+/
            }          
          ]
        },        
      }
    } #:nodoc:

    def protocol(*args)
      if (args.size != 2)
        raise 'too few params'
      end
      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.DelProtocol( {:ticket => res.value, :name => args[1]} )
      end
      return "#{res.value} (#{res.code})"
    end
  
    def host(*args)
      if ((args==nil) || (args.size != 2))
        raise 'too few params'
      end

      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.DelHost( {:ticket => res.value, :id => args[1]} )
      end
      return "#{res.value} (#{res.code})"
    end
    

    def service(*args)
      unless valid_param_count?(2, false, args)
        out = []
        out << 'too few parameters'
        return out+ opt_help('del', 'help', 'service')
      end

      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.DelService( 
          {
          :ticket => res.value, 
          :id => args[1]
          }
        )
      end
      return "#{res.value} (#{res.code})"
    end
    
    def category(*args)
      if (args.size != 2)
        raise 'too few params'
      end
      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.DelCategory( {:ticket => res.value, :name => args[1]} )
      end
      return "#{res.value} (#{res.code})"
    end
    
    #----------------------------------------------------- public methods
    public        
    
  end
end