=begin
**
** edit.rb
** 05/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   This module uses the SOAP interface to edit information that is already in
** the knowledge base of the dradis server.
**
** Version:
**  v1.0 [05/February/2008]: first released
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Commands
  
  # This module uses the SoapDriver to edit information that is already in the 
  # knowledge base of the +dradis+ server. Actions provided are:
  # [+note+] edit the information of a specific note of a host or service
 
  #:nodoc:
  # [+host+] add a new host
  # [+protocol+] add a new IP protocol
  # [+service+] add a new service for a host on a given port
  # [+category+] add a new note category
  class Edit < Dispatchers::Namespace
    include SoapDriver

 
    #----------------------------------------------------- private methods
    private
    INFO = {
      :namespace => 'edit',
      :description => 'edit information already present in the knowledge base of the server',
      :commands => {
#        'host' => {
#          :desc => 'new host',
#          :syntax => [
#          {
#            :required => true,
#            :label=>'host address', 
#            :regexp=> /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/
#          }             
#          ]
#        },
#        'protocol' => {
#          :desc => 'new IP protocol',
#          :syntax => [
#          {
#            :required => true,
#            :label=>'protocol name', 
#            :regexp=> /\w+/
#          }             
#          ]
#        },
#        'service' => {
#          :desc => 'new service for a host on a given port',
#          :syntax => [
#          {
#            :required => true,
#            :label =>'host_name',  
#            :regexp=> /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/
#          },
#          {
#            :required => true,
#            :label => 'service_name', 
#            :regexp=> /\w+/
#          },          
#          {
#            :required => true,
#            :label => 'port', 
#            :regexp=> /\d+/
#          },          
#          {
#            :required => true,
#            :label => 'protocol',
#            :regexp=> /\w+/
#          }              
#          ]
#        },
#        'category' => {
#          :desc => 'new note category',
#          :syntax => [
#          {
#            :required => true,
#            :label=>'category_name', 
#            :regexp=> /\w+/
#          }             
#          ]
#        },
        'note' => {
         :desc => 'note attributes',
          :syntax => [
          {
            :required => true,
            :label =>'note_id',  
            :regexp=> /\d+/
          },
          {
            :required => true,
            :label => 'author', 
            :regexp=> /\w+/
          },          
          {
            :required => true,
            :label => 'category',
            :regexp=> /\w+/
          },
          {
            :required => true,
            :label  => 'text',
            :regexp => //
          }
          ]          
        }
      }
    }#:nodoc:

#    def protocol(*args)
#      if (args.size != 2)
#        raise 'too few params'
#      end
#      setup_soap
#      res = @driver.requestticket
#      if res.code == 0
#        setup_soap
#        res = @driver.AddProtocol( {:ticket => res.value, :name => args[1]} )
#      end
#      return "#{res.value} (#{res.code})"
#    end
#  
#    def host(*args)
#      if ((args==nil) || (args.size != 2))
#        raise 'too few params'
#      end
#      
#      setup_soap
#      res = @driver.requestticket
#      if res.code == 0
#        setup_soap
#        res = @driver.AddHost( {:ticket => res.value, :address => args[1]} )
#      end
#      return "#{res.value} (#{res.code})"
#    end
#
#    def category(*args)
#      if ((args==nil) || (args.size != 2))
#        raise 'too few params'
#      end
#      
#      setup_soap
#      res = @driver.requestticket
#      if res.code == 0
#        setup_soap
#        res = @driver.AddCategory( {:ticket => res.value, :name => args[1]} )
#      end
#      return "#{res.value} (#{res.code})"
#    end
#
#    #add service host_id service_name service_port service_protocol
#    def service(*args)
#      if ((args==nil) || (args.size != 5))
#        raise 'too few params'
#      end
#      
#      setup_soap
#      res = @driver.requestticket
#      if res.code == 0
#        setup_soap
#        res = @driver.AddService( 
#        {
#          :ticket => res.value, 
#          :host => args[1],
#          :name => args[2],
#          :port => args[3],
#          :protocol => args[4]
#        }
#        )
#      end
#      return "#{res.value} (#{res.code})"
#    end
#    
    def note(*args)
      if (args.size != 5)
        raise 'wrong number of arguments. please check your syntax.'
      end            
      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.EditNote(
          {
          :ticket => res.value,
          :uid => args[1],
          :author => args[2],
          :category => args[3],
          :text=> args[4]
          }
        )
      end
      return "#{res.value} (#{res.code})"
    end

    #----------------------------------------------------- public methods
    public        

    
  end
end