=begin
**
** export.rb
** 2 February 2008
** Siebert Lubbe siebertlubbe@googlemail.com
**
** Desc:
** Exports the Dradis knowledge base to a specified file and format
** Version:
** v1.0 [february 2008]
** v1.1 [31/March/2008]: bug fixed - error in passing categories to xml file
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Commands
  # The class presents methods to export the dradis knowledge base data to the client
  # Actions:
  # [+xml+] exports the data to xml format and save it to a file as requested by the user
  class Export < Dispatchers::Namespace
    require 'rexml/document'
    include REXML
    include SoapDriver
    
    #----------------------------------------------------- private methods
    private 
    
    #----------------------------------------------------- Dispatcher implementation
    INFO = {
      :namespace => 'export',
      :description => 'export the knowledge base to a local file',
      :commands => {
        'xml' => {
          :desc => 'export to xml',
          :syntax => [
          {
            :required => true,
            :label => 'file name',
            :regexp => /\w+/
          }
          ]
        }
      }
    }
	
    
    def xml(*args)
      setup_soap
      # get the knowledge base from the server
      server_data = @driver.refresh
      
      # determine the operating system and extract the directory part of the file name given
      if ::Config::CONFIG["host_os"] == "mswin32"	  
        args[1] =~ /^(.*\\)*(.+)$/
      else
        args[1] =~ /^(.*\/)*(.+)$/	
      end
      
      if !$2 
        return "no file name given"
      end
      
      # if no extention is given then add .xml
      if !(args[1] =~ /.*\..+/)
        args[1] = args[1] + ".xml"
      end
      
      # check if the given file is valid
      if  $1 && !File.exists?($1)
        return "#{$1} is not a valid directory"
      end	  
      if  $1 && !File.stat($1).writable?
        return "you do not have write access to #{$1}"
      end
      
      # instantiate a xml object
      xml_doc = Document.new
      xml_doc << XMLDecl.new
      xml_doc.add_element('hosts')                  
      
      # iterate through the knowledge base data and populate the xml object
      server_data.hosts.each do |host|
        xml_host = Element.new('host')
        xml_host.add_element('address').text = host.address
        
        
        xml_services = Element.new('services')
        host.services.each do |service|
          xml_service = Element.new('service')
          xml_service.add_element('name').text = service.name
          xml_service.add_element('protocol').text = service.protocol
          xml_service.add_element('port').text = service.port
          
          xml_service_notes = Element.new('notes')
          service.notes.each do |note|
            xml_service_note = Element.new('note')
            xml_service_note.add_element('author').text = note.author
            xml_service_note.add_element('text').text = note.text
            xml_service_note.add_element('category').text = note.category            
            xml_service_notes << xml_service_note
          end 
          xml_service << xml_service_notes
          
          xml_services << xml_service
        end
        xml_host << xml_services
        
        xml_notes = Element.new('notes')
        host.notes.each do |note|
          xml_note = Element.new('note')
          xml_note.add_element('author').text = note.author
          xml_note.add_element('text').text = note.text
          xml_note.add_element('category').text = note.category
          xml_notes << xml_note
        end
        xml_host << xml_notes
        
        xml_doc.root << xml_host
      end
      
      # write the xml data to the given file name
      xml_file = File.new(args[1],"w")
      xml_doc.write(xml_file)
      xml_file.close
      
      return "knowledge base data saved to #{args[1]}"      
      
    end

    
  end
  
end