=begin
**
** show.rb
** 18/JUN/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Includes actions to query information already stored on the server.
** Version:
**  v1.0 [18/June/2007]: first released
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Commands
  # This module contains SOAP operations to query information already stored
  # on the +dradis+ server. Actions provided are:
  # [+protocols+] display the list of protocols stored in the server
  # [+categories+] display the list of available note categories
  class Show < Dispatchers::Namespace
    include SoapDriver
    
    #----------------------------------------------------- private methods
    private
    #----------------------------------------------------- NamespaceDispatcher interface  
    INFO = { 
      :namespace => 'show',
      :description => 'query information already stored on the server',
      :commands => {
        'protocols' => { 
          :desc => 'display the list of protocols stored in the server', 
          :syntax => []
        },
        'categories' => { 
          :desc => 'display the list of available note categories', 
          :syntax => []
        }
      }
    } #:nodoc:
    
    def protocols(*args)
      out = ['Protocol list:']
      setup_soap
      list = @driver.ShowProtocols
      if (list.size > 0) 
        list.collect do |proto| out << "\t#{proto}" end
      else
        out << "\tempty"
      end
      return out
    end
    def categories(*args)
      out = ['Category list:']
      setup_soap
      list = @driver.ShowCategories
      if (list.size > 0) 
        list.collect do |cat| out << "\t#{cat}" end
      else
        out << "\tempty"
      end
      return out
    end   
    #----------------------------------------------------- public methods
    public
  end
end