=begin
**
** soap.rb
** 19/JUN/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Contains SOAP operations that do not fit in the Add/Del/Show categories
** Version:
**  v1.0 [19/June/2007]: first released
**  v1.1 [05/February/2008]: store the results pulled from the server in the
**    $model object.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'soap/wsdlDriver'
require 'soap/netHttpClient'
SOAP::HTTPStreamHandler::Client = SOAP::NetHttpClient
SOAP::HTTPStreamHandler::RETRYABLE = false

module Commands 
  # This module contains SOAP operations that do not fit in the Add/Del/Show
  # categories. Actions provided are:
  # [+changed?+] check if the local revision of the knowledge base is the latest
  #              available in the server.
  # [+refresh+] get the latest knowledge base from the server.
  class Soap < Dispatchers::Simple
    include SoapDriver
    
    #----------------------------------------------------- private methods
    private
    INFO = { 
      :commands => {
        'changed?' => {
          :desc => 'Check if the local revision of the knowledge base is the latest available in the server.',
          :syntax => [
            {
              :required => true,
              :label=>'revision number', 
              :regexp=> /\d+/
            }          
          ]
        },
        'refresh' => {
          :desc => 'Get the latest knowledge base from the server.',
          :syntax => [
            {
              :required => true,
              :label=>'revision number', 
              :regexp=> /\d+/
            }          
          ]
        }
      }
    } #:nodoc:
    attr :last_revision
    
    def changed?(*args)
      if ((args==nil) || (args.size != 2)) 
        raise 'too few params'
      end

      setup_soap
      @driver.changed args[1]
    end
    
    def refresh(*args)
      if ((args==nil) || (args.size != 2)) 
        raise 'too few params'
      end    
      
      # TODO: is not clear what I should return if no change was made
      return nil unless changed?(nil, args[1])
      
      setup_soap
      $model.refresh_with( @driver.refresh )
      return "update completed. local revision #{$model.revision}"
    end
    
    #----------------------------------------------------- public methods
    public
  end
end