=begin
**
** context.rb
** 30/JUL/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   One of the important pieces of the architecture, this file contains the 
** definition of the Controller, Service and View interfaces.
**
** Version:
**  v1.0 [30/July/2007]: first released
**  v1.1 [01/February/2008]: add more methods to the ViewInterface:
**           - exec
**           - constructor that accepts command line parameters
**
**  v1.2 [05/February/2008]: 
**           - add the 'parse_command' method to the controller.
**           - create a new interface for the Model
**           - rename 'exec' for 'run' in the View interface, it was causing
**           some issues with the Qt interface.
**           - re-enable notes_for, add_note, edit_note, refresh_model methods 
**           of the controller interface.
**  v1.3 [06/February/2008]: add a new method (teardown) to the View interface,
**           from now on, the controller will capture the Core::DradisQuitApp
**           exception and will notify the view and exit.
**  v1.4 [20/February/2008]: add find_note to the controller interface to be
**           used by the view.
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core

  # Interface for the Model object. The model is responsible for storing all
  # the relevant information in the server's knowledge base.
  module ModelInterface
    def refresh_with(new_model) raise 'unimplemented!' end
    
    def revision() raise 'unimplemented!' end
    def hosts() raise 'unimplemented!' end
    def categories() raise 'unimplemented!' end
    def protocols() raise 'unimplemented!' end
  end
  
  # Interface for the Service layer. The controller will request operation to
  # be performed in the Model layer through the Service.
  module ServiceInterface
    def dispatch(cmd, *args) raise 'unimplemented!' end
    def model() raise 'unimplemented!' end
    
    # TODO: old functionality, review if needed
    def refresh_model() raise 'unimplemented!' end
    def get_categories() raise 'unimplemented!' end
    def add_note(type,id,author,cat,text) raise 'unimplemented!' end
    def edit_note() raise 'unimplemented!' end
  end
  
  # The Controller will connect the View with the Service layer. This interface
  # provides the methods available to be called by the View.
  module ControllerInterface
    def initialize(params={}) raise 'unimplemented!' end
    def exec() raise 'unimplemented!' end
    def invoke(cmd, *args) raise 'unimplemented!' end
    def model() raise 'unimplemented!' end

    def notes_for(annotatable_type, annotatable_id) raise 'unimplemented!' end
    
    def add_note(annotatable_type,annotatable_id,author,category,text) raise 'unimplemented!' end
    def edit_note(note_id,author,category,text) raise 'unimplemented!' end    
    def find_note(note_id) raise 'unimplemented!' end
    
    def parse_command(command_line) raise 'unimplemented!' end
    def refresh_model() raise 'unimplemented!' end    
  end
  
  # In order to create a new View, this simple interface must be followed.
  module ViewInterface
    # TODO: fix the constructor implementation issue Qt vs. WxWidgets
    #def initialize(argv) raise 'unimplemented!' end
    def setup(new_controller) raise 'unimplemented!' end
    
    # When the service layer raises a Core::DradisQuitApp exception, the 
    # controller will it and notify the view by invoking this method.
    def teardown() raise 'unimplemented!' end
    def run() raise 'unimplemented!' end
  end
end

require 'core/model'
require 'core/controller'
require 'core/service'

