=begin
**
** service.rb
** 03/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   One of the important pieces of the architecture, this file contains the 
** implementation of the Service interface. The Service makes all the 
** functionality of the different modules available to the controller.
**
** Version:
**  v1.0 [03/Feb/2008]: first released
**  v1.1 [05/Feb/2008]: instantiate a Model object that will be used to
**            store all the knowledge pulled from the server.
**  v1.2 [22/Feb/2008]: implement the 'reload' functionality to dinamically
**            reload the modules available.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'commands'

module Core
  
  # TODO: move the Dispatcher module inside Core
  # This class is inherits from Dispatchers::Simple because it provides some
  # actions to the user.
  class Service < Commands::Dispatchers::Simple
    include Core::ServiceInterface
    attr_accessor :dispatchers
    # TODO: attr :model    
  
    #----------------------------------------------------- private methods
    private
    
    #----------------------------------------------------- Dispatcher implementation
    # Returns a Hash of commands supported by this Dispatcher. See 
    # Dispatcher#commands for output format.
    # 
    # The current list of supported commands for this Dispatcher is:-
    # [quit] Exits the application
    # [help] Shows the help of all the commands available
    INFO = { 
      :commands => {
        # TODO: does it make sense to have a 'clear' command here? wouldn't it
        # be better if the Parser of the View has some 'dispatching' 
        # functionality of this kind?
        'clear' => {
          :desc => 'clean the console window',
          :syntax => []#no params
        },
        'reload' => {
          :desc => 'reload the modules from the filesystem',
          :syntax => []#no params
        },
        'debug' => {
          :desc => 'various debuging information',
          :syntax => []#no params
        },
        'help' => { 
          :desc => 'shows the help of all the commands available', 
          :syntax => []#no params
        },
        # TODO: does it make sense to have a 'quit' command here? wouldn't it
        # be better if the Parser of the View has some 'dispatching' 
        # functionality of this kind?        
        'quit' => { 
          :desc => 'exits the application', 
          :syntax => []#no params
        }
      }
    }

    # This method will provide a way to clear the console (cli or gui) of
    # +dradis+ however is not implemented :(
    def clear(*args)
      raise '*clear* unimplemented!'
    end     

    # Action to close the client. It is implemented by raising a 
    # Core::DradisQuitApp exception.
    def quit(*args)
      raise Core::DradisQuitApp
    end
    
    # Handle +help+ requests. A list of available commands are returned.
    def help(*args)
      commands = ['List of registered commands']
      commands << '---------------------------'
      self.dispatchers.each do |d|
        next if !d.respond_to?('commands') ||
        d.commands == nil || 
        d.commands.size == 0
                
        d.commands.each do |cmd, opts| commands << "#{cmd}:\t#{opts[:desc]}" end
      end
      commands
    end    
    
    # Helper function to retrieve the list of registered dispatchers.
    def debug(*args)
      out = ['List of dispatchers']
      out << '---------------------------'      
      self.dispatchers.each do |d|
        out << "#{d.class.to_s} < #{d.class.superclass.to_s}"
      end 
      return out
    end

    # Creates an instance of the parser. All the dispatchers located under 
    # the directory <tt>/commands/modules/</tt> are loaded and instanced.
    def reload(*args)
      self.dispatchers = [self]
      Dir.entries('commands/modules/').each do |dispatcher|
        next if File.extname(dispatcher) != '.rb'
        klass = File.basename(dispatcher, '.rb')
        load "commands/modules/#{dispatcher}"
        self.dispatchers << eval('Commands::'+klass.capitalize!).new
      end
      self.dispatchers.collect do |d| d.class.to_s end
    end    
    
    #===================================================== public methods
    public
    
    # Creates an instance of the parser. All the dispatchers located under 
    # the directory <tt>/commands/modules/</tt> are loaded and instanced.
    def initialize
      reload
      
      # TODO: think about this, there should be a better way!!
      $model = Core::Model.new
    end
    
      
    #
    # For a given command (_cmd_), this method walks through the registered
    # dispatchers to see if anyone is able to run the command. If a valid 
    # dispatcher is found, the _run_ method for this dispatcher is called 
    # passing along the command name and parameters entered by the user.
    #
    def dispatch(cmd, *args)
      $logger.debug {"dispatching command at #{self.class.to_s}"}
      $logger.debug {cmd}
      $logger.debug {args}
      
      found = false
      out = "command <b>#{cmd}</b> not implemented"
      self.dispatchers.each do |d|
        #         $logger.info {d.class.to_s}
        #         $logger.info {d.commands}
        next if !d.respond_to?('commands') ||
        d.commands == nil || 
        d.commands.size == 0
        
        $logger.debug {d.class.to_s}
        #$logger.debug {d.commands}
        if (d.commands.has_key? cmd)
          $logger.debug{d.commands.keys}
          #out = d.send('cmd_' + cmd, *params)
          out = d.run(cmd, *args)
          found = true
        end
        break if found
      end
      
      if out.instance_of?(Array)
        return out.join("\n")
      else
        return out
      end      
    end

    def model
      $model
    end

  end
end