=begin
**
** gui.rb
** 02/JUN/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  This script module contains all the User Interface related code
**
** Version:
**  v1.0 [02/Jun/2007]: first released
**  v1.1 [21/Jun/2007]: new module based architecture
**  v1.2 [05/Feb/2008]: after creating the WxWidgets interface, some changes
**     in the architecture were introduced to follow the MVC pattern, some
**     modifications are needed for all the Qt based classes.
**     - improvements in the way the 'Qt4' library is required.
**  v1.3 [06/Feb/2008]: 
**     - try to find the Qt bindings harder . First check for 'Qt4'. if that 
**     doesn't work, try 'Qt'. if that doesn't work either, give up :(
**     - implement the new teardown() method of the View interface.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

begin
  require 'Qt4'
rescue LoadError => no_qt4
  # okay, try Qt
  begin
    require 'Qt'
  rescue LoadError => no_qt4
    # :-/ no hope here
    $logger.debug no_qt4.message
    $logger.error('Qt4 bindings for ruby not found. install them using: apt-get install libqt4-ruby')
    raise no_qt4
  end
end

require 'ui/qt/window'

# Different user interfaces will be grouped under this module.
module Ui
  # Graphical user interface classes and modules. Based on version 4 of the 
  # Qt library (see http://doc.trolltech.com/4.0/).
  module Qt4
    # Main Qt Application, handles the creation of main window, window title, etc.
    class DradisGui < Qt::Application
      include Core::ViewInterface
      
      # Internal setup of the interface elements. Main application calls this
      # method to instantiate the command parser, window title, etc.
      def setup(new_controller)
        @w = Ui::Qt4::DradisWindow.new
        @w.windowTitle = 'dradis'
        @w.controller = new_controller
        @w.show
      end
      
      # Show the main application window
      def run
        self.exec
      end
      
      def teardown
        self.quit()
      end
    end
    
  end #module Qt4
end  #module Ui

