=begin
**
** wx.rb
** 01/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  This script module contains all the User Interface related code using the 
** wxWidgets implementation
**
** Version:
**  v1.0 [01/Feb/2008]: first released
**  v1.1 [06/Feb/2008]: 
**     - improve the error handling when loading the wx gem
**     - implement the teardown() method of the View interface
**     - adapt the exec() method to run() as the new view interface requires
**  v1.2 [19/Feb/2008]: add references to the new dialog interfaces
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

# first check that the gem is installed, if it is not, provide some helpful
# clue
begin
  require 'wx'
rescue LoadError => no_wx_err
  begin
    require 'rubygems'
    require 'wx'
  rescue LoadError => no_wx_err
    $logger.debug no_wx_err.message
    $logger.error('wxruby gem not found. install it using: gem install wxruby')
    raise no_wx_err
  end
end

require 'ui/dialogs'
require 'ui/widgets'
require 'ui/wx/window'

# Different user interfaces will be grouped under this module.
module Ui
  # Graphical user interface classes and modules. Based on the wxWidgets
  # framework (http://www.wxwidgets.org/). In order for this to work you will 
  # need the wxruby gem (http://wxruby.rubyforge.org/) installed.
  module WxWidgets
    # Main Wx Application, handles the creation of main window, window title, etc.
    class DradisGui < Wx::App
      include Core::ViewInterface
      
      def on_init
        @frame = Ui::WxWidgets::DradisWindow.new( nil, 
                           Wx::ID_ANY, 
                           "dradis",
                           Wx::DEFAULT_POSITION,
                           Wx::Size.new(800, 600) 
                        )
        set_top_window(@frame)
        @frame.controller = @controller
        @frame.show
        return true                       
      end
      def setup(new_controller)
        @controller = new_controller
      end
      def run()      
        main_loop
      end
      def teardown()
        @frame.close 
      end
    end
  end
end
