=begin
**
** note.rb
** 19/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  wxWidgets implementation of the NoteDialogInterface
**
** Version:
**  v1.0 [19/February/2008]: first released
**  v1.1 [20/February/2008]: go for a XML based interface. we can define the 
**          graphical elements in an interface builder such as DigitalBlocks and 
**          then use ruby code to define events and other business logic.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Ui
  module WxWidgets
    module Dialogs
      
      class NoteDialog < Wx::Dialog
        include NoteDialogInterface
        #===================================================== private methods
        private
        
        #===================================================== public methods
        public    

        def initialize(parent=nil)
          super()
          
          # TODO: think about a way of sharing this XmlResource between 
          # different wx components
          xml = Wx::XmlResource.get();
          xml.init_all_handlers();
          xml.load("ui/wx/dialogs/note.xrc")
          
          #
          # Load the dialog from XRC. We define $xml in on_init.
          # We could use XmlResource.get() over and over again, but
          # honestly, thats just too much work.
          #
          xml.load_dialog_subclass(self,parent,'ID_NOTEDIALOG')
		
          #
          # Get the buttons. Note we use 'xrcid' much like the XRCID macro
          # in C++. Make sure the id's actually exist, or your program will crash!
          #
          @author = Wx::Window.find_window_by_id(Wx::xrcid('ID_AUTHOR'),self)
          @category = Wx::Window.find_window_by_id(Wx::xrcid('ID_CATEGORY'),self)
          @text = Wx::Window.find_window_by_id(Wx::xrcid('ID_TEXT'),self)
          
          #
          # Bind the buttons to event handlers
          #
          evt_button(Wx::xrcid('wxID_OK')) do
            end_modal(Wx::ID_OK)
          end
		
          evt_button(Wx::xrcid('wxID_CANCEL')) do
            end_modal(Wx::ID_CANCEL)
          end      
        end

        def author() 
          @author.get_value
        end
        def category() 
          @category.get_value
        end
        def text() 
          @text.get_value
        end
    
        def categories=(category_list)
          @category.clear
          category_list.each do |c|
            @category.append c.name
          end
          @category.set_selection(0)
        end
        
        def edit_note(note)
          @author.set_value(note.author)
          @text.set_value(note.text)
          @category.set_value(note.category)
        end
      end
    end
  end
end
