=begin
**
** hoststree.rb
** 03/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  wxWidgets implementation of the HostsBrowserInterface
**
** Version:
**  v1.0 [03/Feb/2008]: first released
**  v1.1 [05/February/2008]: disable the tree view while content is loading, 
**            re-enable after the update is completed. Don't populate with
**            dummy content on start up.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Ui
  module WxWidgets
    module Widgets
      class HostsBrowser < Wx::TreeCtrl
        include HostsBrowserInterface
        
        #===================================================== private methods
        private

        def setup_images
          img_list = Wx::ImageList.new(16, 16, true, 2)
          img_list.add( 
            Wx::ArtProvider::get_bitmap( 
              Wx::ART_FOLDER,
              Wx::ART_OTHER,
              Wx::Size.new(16, 16) 
            ) 
          )
          img_list.add( 
            Wx::ArtProvider::get_bitmap( 
              Wx::ART_NORMAL_FILE,
              Wx::ART_OTHER,
              Wx::Size.new(16, 16) 
            ) 
          )
          
          self.set_image_list(img_list)          
        end
        def setup_dummy_content
           root = self.add_root("Wx::AUI Project", 0)
           items = []
           1.upto(5) { | i | items << self.append_item(root, "Item #{i}", 0) }
   
           items.each do | id |
             1.upto(5) { | i | self.append_item(id, "Subitem #{i}", 0) }
           end
   
           self.expand(root)          
        end
        #===================================================== public methods
        public
        
        def initialize(parent=nil)
          super(parent, 
            Wx::ID_ANY, 
            Wx::DEFAULT_POSITION,
            Wx::Size.new(200, 250),
            Wx::TR_DEFAULT_STYLE|Wx::NO_BORDER|Wx::TR_HIDE_ROOT)
          
          setup_images
          #setup_dummy_content
          self.disable
        end
        
        def update_view(model)
          self.delete_all_items
          
          root = self.add_root('dradis', 0)
          model.hosts.each do |host|
            parent = self.append_item(root, "#{host['uid']}: #{host['address']}", 0)
            self.set_item_data(parent, {:type=>:host, :id => host.uid, :notes => host.notes})
            
            host.services.each do |srv|
              item = self.append_item(parent, "#{srv.uid}: #{srv.name} (#{srv.protocol}/#{srv.port})", 1)
              self.set_item_data(item, {:type=>:service, :id => srv.uid, :notes => srv.notes})
            end
          end
          self.expand_all
          self.enable
        end
      end
    end
  end
end