=begin
**
** notesbrowser.rb
** 03/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  wxWidgets implementation of the NotesBrowserInterface.
**
** Version:
**  v1.0 [03/February/2008]: first released
**  v1.1 [05/February/2008]: play with the 'evt_html_link_clicked' event to
**            see how it works
**  v1.2 [19/February/2008]: add processing logic to the 'evt_html_link_clicked' 
**            to emit signals when adding/editing notes and filter notes.
**  v1.3 [20/February/2008]: 
**            - implement the invalidate!() function as mandated by the 
**            NotesBrowserInterface and required when the user is editing
**            a note.
**            - implement the html_src functionality to keep track of the latest
**            html code loaded.
**  v1.4 [03/April/2008]: escape HTML entities in the notes text using 
**            CGI::escapeHTML
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'cgi' # for escaping HTML entities

module Ui
  module WxWidgets
    module Widgets
      class NotesBrowserEvent < Wx::CommandEvent
        # Create a new unique constant identifier, associate this class
        # with events of that identifier, and create a shortcut 'evt_new_command'
        # method for setting up this handler.
        EVT_ADD_NOTE = Wx::EvtHandler.register_class(self, nil, 'evt_add_note', 1)
        EVT_EDIT_NOTE = Wx::EvtHandler.register_class(self, nil, 'evt_edit_note', 1)

        def initialize(notes_browser, event_id, annotatable_type, annotatable_id, note_uid = nil)
          # The constant id is the arg to super
          super(event_id)
          # client_data should be used to store any information associated
          # with the event.
          self.client_data = { 
            :annotatable_type => annotatable_type,
            :annotatable_id => annotatable_id,
            :uid => note_uid
            }
          self.id = notes_browser.get_id
        end

        def annotatable_type
          client_data[:annotatable_type]
        end          
        def annotatable_id
          client_data[:annotatable_id]
        end          
        def note_uid
          client_data.fetch(:uid, nil)
        end          

      end

      class NotesBrowser < Wx::HtmlWindow
        attr_reader :html_src
        
        include NotesBrowserInterface
        
        #===================================================== private methods
        private
        
        def get_html_bar(type, uid, categories)
          cats =  ''
          categories.each do |cat|
            cats << " <a href=\"filter/#{cat}\">#{cat}</a>"
          end
          HTML[:bar].sub(/#type#/,type.to_s).sub(/#uid#/,uid.to_s).sub(/#catlist#/, cats)
        end
        def filter_and_display(filter='all')
          categories = {}
          
          @notes.each do |note|
            categories[note.category] = [] unless categories.has_key?(note.category)
            categories[note.category] << note
          end

          
          html = get_html_bar(@type, @id, categories.keys)
          if @notes.size == 0
            html += HTML[:home]
          else
            categories.each do |name, notes|
              next unless ((filter=='all')||(filter==name))
              html << "<div><h3>#{name}</h3><ul>"
              notes.each do |n|
                extra = ''
                text = n.text
                if (text.size > 60)
                  text = text[0,59] << '...'
                  extra = "[<a href=\"show/#{n.uid}\">details</a>]"
                end
                html << "<li>#{CGI::escapeHTML(text)} (<em>#{n.author}</em>) #{extra}[<a href=\"edit/#{n.uid}\">edit</a>]</li>"
              end
              html << '</ul></div>'
            end
          end
          self.set_page( html )
          @html_src = html
        end
        
        
        def on_link_clicked(event)
          url = event.get_link_info.get_href 
          $logger.debug{"showing url: #{url}"}

          if url == '#'
            self.set_page( @last_html )
          else
            params = url.split('/')
            
            case params[0].intern
            when :filter
              filter_and_display(params[1])
            when :show
              @last_html = @html_src
              @notes.each do |note|
                next unless note.uid == params[1].to_i
                extra = "[<a href=\"#\">back</a>]"
                self.set_page( "<div><h3>#{note.category}</h3> <pre>#{CGI::escapeHTML(note.text)}</pre> <p>(<em>#{note.author}</em>) #{extra}[<a href=\"edit/#{note.uid}\">edit</a>]</p></div>" )
              end
            when :add
              event_handler.process_event( NotesBrowserEvent.new(self, NotesBrowserEvent::EVT_ADD_NOTE, @type, @id) )
            when :edit
              event_handler.process_event( NotesBrowserEvent.new(self, NotesBrowserEvent::EVT_EDIT_NOTE, @type, @id, params[1].to_i) )
            end
          end          
        end
        #===================================================== public methods
        public         
        
        def initialize(parent=nil)
          super(parent, Wx::ID_ANY, Wx::DEFAULT_POSITION, Wx::Size.new(400, 300) )
          
          evt_html_link_clicked(self.get_id) { |event| on_link_clicked(event)}
          home
        end

        def set_page(html)
          super html
          @html_src = html
        end
        
        def home()
          self.set_page(HTML[:home])
        end
        
        def show_notes(annotatable_type, annotatable_id, note_list)
          @type = annotatable_type
          @id = annotatable_id
          @notes = note_list
          
          filter_and_display()
        end        
        
        def invalidate!() self.set_page(HTML[:invalid]); end
      end
    end
  end
end
